#!/usr/bin/env python

# busy cursor
# timeout on dns lookups
# too many stars in traceroute


from Tkinter import *

import DNS
import os
import Pmw
import popen2
import re
import select
import socket
import string
import sys
import time

kFileLineSep=os.linesep # Contantize this for speed, global lookup faster than module data lookup

kInterestingForeground="blue"
kTkDefaultBG="#d9d9d9"
kTooLongForTracerouteLine=10 # Seconds
kTracerouteMaxStarLines=3
kMyFont="neep-alt-12"
kMyName="Spamgrep"
kScrollbarWidth=16
kUnHexedBackground="yellow"
kWidgetLineSep="\n" # Tkinter always uses newlines, it seems

kInterestingStuffREs= \
  {
  # Mostly not syntactically exactly correct but should be sufficient for our purposes.
  'emailDomain': re.compile("@([-a-zA-Z0-9.]+\.[-a-zA-Z0-9]+)"),
  'urlDomain1':  re.compile("http://([-a-z0-9.]+)",re.IGNORECASE), # Domain part of a URL
  'urlDomain2':  re.compile("(www.[-a-z0-9.]+)",re.IGNORECASE),
  'dottedQuad':  re.compile("([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})"),
  'decimalIP':   re.compile("http://([0-9]{8,10})"),
  'usPhoneNo':   re.compile("(\({0,1}[1-9][0-9]{2}\){0,1}[-. ]{0,1}[1-9][0-9]{2}[-. ][0-9]{4})"),
  }

heloRE=re.compile("from ([^ ]+)")
parensRE=re.compile("\((.*?)\)")
reverseLookupRE=re.compile("^[^[ ]*")
ipRE=re.compile("\[(.*)\]")
byRE=re.compile("by ([^ ]+)")

hexEscapeRE=re.compile("%([0-9a-fA-F][0-9a-fA-F])")
decEscapeRE=re.compile("&#([0-2]{0,1}[0-9][0-9]);")

# Could do a thing like traceroute from within Python but we'd need to
# run as root to open the raw socket.  Don't want to do that.
whichTraceroute="/usr/sbin/traceroute"

# Barf, I hate regular expressions. Even for something simple like
# this, it gets ugly: at the beginning of a line, skip any number of
# spaces, any run of 1 or more digits, any number of stars or spaces,
# then find a run of letters, numbers, dashes and dots, and keep track
# of that.
hostNameRE=re.compile("^ *[0-9]+[* ]*([-a-zA-Z0-9.]+)")

# Code here swiped from http://www.dotfunk.com/projects/whois/ and
# modified by me.  My thanks to the author.

def doWhois(query,whoisserver):
  s=None
  # Try until we're connected or 5 seconds has elapsed.
  start=time.time()
  while s == None and time.time()-start<5:
    try:
      s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      s.setblocking(0)
      try:
        s.connect((whoisserver, 43))
      except socket.error, (ecode, reason):
        if ecode in (115, 150): pass
        else:
          raise socket.error, (ecode, reason)
      ret = select.select([s], [s], [], 30)

      if len(ret[1])== 0 and len(ret[0]) == 0:
        s.close()
        raise TimedOut, "on connect "
      s.setblocking(1)

    except socket.error, (ecode, reason):
      time.sleep(10)
      s = None

  if s==None:
    return None

  s.send("%s \n\n" % query)
  page = ""
  while 1:
    data = s.recv(8196)
    if not data: break
    page = page + data

  s.close()
  return page

def doWhoisIP(ip,outputWin):
  arinNotHereRE=re.compile("Domain System inverse mapping provided by.*ripe.net", \
    re.MULTILINE|re.DOTALL|re.IGNORECASE)
  arinMultiDelegationRE=re.compile("^To single out one record",re.MULTILINE)
  arinHandleRE=re.compile("\((.*?)\)")
  ripeNotHereRE=re.compile("This address space is assigned at various other places",re.MULTILINE)

  queryResult=doWhois(ip,"whois.arin.net")

  if queryResult==None:
    outputWin.addLine("Server whois.arin.net not answering.%s" % kWidgetLineSep)
    return None

  outputWin.addGulp(queryResult)

  if not arinNotHereRE.search(queryResult):
    # ARIN has the info
    if arinMultiDelegationRE.search(queryResult):
      handles=arinHandleRE.findall(queryResult)
      for handle in handles:
        outputWin.addLine("")
        queryResult=doWhois("!%s" % handle,"whois.arin.net")
        if queryResult==None:
          outputWin.addLine("Server whois.arin.net not answering.%s" % kWidgetLineSep)
          return None
        outputWin.addGulp(queryResult)
  else:
    # try RIPE
    outputWin.addLine("")
    queryResult=doWhois(ip,"whois.ripe.net")
    if queryResult==None:
      outputWin.addLine("Server whois.ripe.net not answering.%s" % kWidgetLineSep)
      return None
    outputWin.addGulp(queryResult)

    if ripeNotHereRE.search(queryResult):
      # try APNIC
      outputWin.addLine("")
      queryResult=doWhois(ip,"whois.apnic.net")
      if queryResult==None:
        outputWin.addLine("Server whois.apnic.net not answering.%s" % kWidgetLineSep)
        return None
      outputWin.addGulp(queryResult)

  return None

def doWhoisDomain(name,outputWin):
  internicNotHereRE=re.compile("no match for",re.IGNORECASE)
  internicReferralRE=re.compile("^\s*whois server:\s*([-a-z0-9.]+)",re.MULTILINE|re.IGNORECASE)
  domainEmailRE=re.compile("[^ ]+@[-a-zA-Z0-9.]+")
  nsiRegistrantRE=re.compile("registrant:\012([^(]*)",re.IGNORECASE)

  queryResult=doWhois(name,"whois.internic.net")
  if queryResult==None:
    outputWin.addLine("Server whois.internic.net not answering.%s" % kWidgetLineSep)
    return None
  outputWin.addGulp(queryResult)
  if not internicNotHereRE.search(queryResult):
    referredTo=internicReferralRE.search(queryResult).group(1)
    queryResult=doWhois(name,referredTo)
    if queryResult==None:
      outputWin.addLine("Server %s not answering.%s" % (referredTo,kWidgetLineSep))
      return None
    outputWin.addGulp(queryResult)
  return None

def forwardLookup(name,outputWin):
  outputWin.addLine("Sending query...")
  ips=[]
  requestor=DNS.Request()
  retVal=requestor.req(name)
  if len(retVal.answers)==0:
    outputWin.clear()
    outputWin.addLine("No reply")
    return None
  else:
    for item in retVal.answers:
      if item['typename']=="A":
        ips.append(item['data'])
    outputWin.clear()
    outputWin.addLine(string.join(ips,", "))
    return ips

def reverseLookup(ip,outputWin):
  outputWin.addLine("Sending query...")
  try:
    ans=DNS.revlookup(ip)
    outputWin.clear()
    outputWin.addLine(ans)
    return ans
  except IndexError:
    outputWin.clear()
    outputWin.addLine("No reply")
    return None

def doMXLookup(orgDomain,outputWin):
  outputWin.addLine("Sending query...")
  serverList=DNS.mxlookup(orgDomain)
  outputWin.clear()
  if len(serverList)==0:
    outputWin.addLine("None")
  else:
    serverList.sort()
    outputList=[]
    requestor=DNS.Request()
    for serverTuple in serverList:
      server=serverTuple[1]
      bunchOfData=requestor.req(server).answers
      ipList=[]
      for item in bunchOfData:
        if item['typename']=="A":
          ipList.append("["+item['data']+"]")
      outputList.append(server+" "+string.join(ipList,", "))
    ansStr=string.join(outputList,", ")
    outputWin.addLine(ansStr)
  return None

def thisIsAnIP(str):
  bytes=string.split(str,".")
  if len(bytes)<>4:
    return None
  for byte in bytes:
    try:
      intVal=int(byte)
    except ValueError:
      return None
    if intVal<0 or intVal>255:
      return None
  return 1

class lookupWin:
  def __init__(self):
    self.commandHist=[]
    self.createWidgets()
    return None

  def show(self):
    self.myWin.deiconify()
    gTkRoot.update_idletasks()
    return None

  def createWidgets(self):
    myWin=Toplevel()
    myWin.title(kMyName+" lookup")
    myWin.protocol("WM_DELETE_WINDOW",self.closeWinCB)
    int=Pmw.PanedWidget(myWin,orient="vertical")
    int.add('top',size=100)
    int.add('bottom')
    int.pack(side=TOP,fill=BOTH,expand=1)
    slb=Pmw.ScrolledListBox(int.pane('top'),vscrollmode="static",selectioncommand=self.changeOutputText)
    slb.component("listbox").configure(font=kMyFont,bg="white")
    slb.component("vertscrollbar").configure(activebackground=kTkDefaultBG)
    slb.pack(side=TOP,fill=BOTH,expand=1)
    self.myListbox=slb
    myText=Pmw.ScrolledText(int.pane('bottom'),text_wrap='word')
    myText.component("text").configure(bg="white",font=kMyFont,state=DISABLED)
    myText.component("vertscrollbar").configure(activebackground=kTkDefaultBG)
    myText.pack(side=TOP,fill=BOTH,expand=1)
    myWin.bind('<Key>',self.keyCB)
    self.myWin=myWin
    self.myText=myText
    self.myWin.withdraw()
    return None

  def lift(self):
    self.myWin.lift()
    return None

  def changeOutputText(self):
    sel=self.myListbox.curselection()
    try:
      sel = map(int, sel)
    except ValueError: pass
    assert len(sel)==1
    self.myText.component("text").configure(state=NORMAL)
    self.myText.clear()
    self.myText.settext(self.commandHist[sel[0]]['output'])
    self.myText.component("text").configure(state=DISABLED)
    return None

  def closeWinCB(self):
    self.myWin.withdraw()
    return None

  def keyCB(self,event):
    if event.char=="w":
      self.closeWinCB()
    return "break"

  def doCommand(self,kind,text):
    self.myText.clear()
    self.show()

    newText=fixUpObfuscatedIP(text)
    if newText<>None:
      obfuscatedText=text
      text=newText
      commandHistItem={'command': "%s %s (%s)" % (kind,text,obfuscatedText),'output': None}
    else:
      if kind=="whois" and not thisIsAnIP(text):
        text=getOrganizationDomain(text)
      if kind=="mx lookup":
        text=getOrganizationDomain(text)
      commandHistItem={'command': "%s %s" % (kind,text),'output': None}

    self.commandHist.append(commandHistItem)
    self.myListbox.setlist(list(self.myListbox.get())+[commandHistItem['command']])
    self.myListbox.select_set(len(self.commandHist)-1)
    gTkRoot.update_idletasks()

    if kind=="nslookup":
      if thisIsAnIP(text):
        reverseLookup(text,self)
      else:
        forwardLookup(text,self)
    elif kind=="whois":
      if thisIsAnIP(text):
        doWhoisIP(text,self)
      else:
        d=getOrganizationDomain(text)
        doWhoisDomain(getOrganizationDomain(text),self)
    elif kind=="traceroute":
      doTraceroute(text,self)
    elif kind=="nslookup, whois":
      if thisIsAnIP(text):
        fqdn=reverseLookup(text,self)
        if fqdn<>None:
          doWhoisDomain(getOrganizationDomain(fqdn),self)
      else:
        ips=forwardLookup(text,self)
        if ips<>None:
          doWhoisIP(ips[0],self)
    elif kind=="mx lookup":
      doMXLookup(text,self)
    else:
      raise CantHappenError("Label menu kind is unknown: %s" % kind)
    commandHistItem['output']=self.myText.get()
    return None

  def addLine(self,line,highlight=0):
    thisLineNo=int(string.atof(self.myText.index(END)))-1
    self.myText.component("text").configure(state=NORMAL)
    self.myText.insert(END,line)
    if line=="" or line[-1]<>kWidgetLineSep:
      self.myText.insert(END,kWidgetLineSep)
    if highlight:
      self.myText.tag_add("highlight","%i.0" % thisLineNo,"%i.999" % thisLineNo)
    self.myText.component("text").configure(state=DISABLED)
    self.myText.yview(sys.maxint)
    self.myWin.update_idletasks()
    return None

  def addGulp(self,gulp):
    self.myText.component("text").configure(state=NORMAL)
    self.myText.insert(END,gulp)
    if gulp=="" or gulp[-1]<>kWidgetLineSep:
      self.myText.insert(END,kWidgetLineSep)
    self.myText.component("text").configure(state=DISABLED)
    self.myText.yview(sys.maxint)
    self.myWin.update_idletasks()
    return None

  def clear(self):
    self.myText.clear()
    return None

def leftMatchCI(longStr,shortStr):
  if len(longStr)<len(shortStr):
    return 0
  return string.lower(shortStr)==string.lower(longStr[0:len(shortStr)])

def fixUpObfuscatedIP(str):
  # Is it a decimal number?
  minIP=0x03000000L # As of now (12/2000) ARIN says that 1/8-2/8 have not been allocated
  maxIP=0xfeffffffL # Actually, nothing above 239/8 has been allocated
  if isAllDigits(str) and long(str)>=minIP and long(str)<=maxIP:
    numVal=long(str)
    return "%i.%i.%i.%i" % \
      (int(numVal>>24), int(numVal>>16 & 0xff),
       int(numVal>>8 & 0xff), int(numVal & 0xff))

  # Maybe octal?
  quad=string.split(str,".")
  if len(quad)==4:
    octal=1
    for octetStr in quad:
      if not isAllOctalDigits(octetStr) or octetStr[0]<>"0":
        octal=0
        break

    if octal:
      for count in range(4):
        if quad[count][0]=="0":
          quad[count]=octStrToDecStr(quad[count])
      return "%s.%s.%s.%s" % tuple(quad)

  # Dunno.
  return None

def octStrToDecStr(octStr):
  decVal=0
  for char in octStr:
    assert char in string.octdigits
    decVal=decVal*8+int(char)
  return str(decVal)

def hexStrToDecVal(hexStr):
  decVal=0
  hexStr=string.lower(hexStr)
  for char in hexStr:
    assert char in string.hexdigits
    decVal=decVal*16+string.find("0123456789abcdef",char)
  return decVal

def isAllOctalDigits(str):
  return filter(lambda char: char in string.octdigits,str)==str

def isAllDigits(str):
  return filter(lambda char: char in string.digits,str)==str

# Prolly exceptions, add them as we encounter them.
def getOrganizationDomain(fqdn):
  fqdn=string.lower(fqdn)
  items=string.split(fqdn,".")
  if len(items)==1:
    return fqdn # Wasn't an FQDN, was it.
  if items[-1] in ["com","net","org","gov","mil"]:
    return string.join(items[-2:],".")
  if len(items[-1])==2:
    if items[-2] in ["edu","com","net","org","co","ne","ad","go","ac","or"]:
      return string.join(items[-3:],".")
    else:
      return string.join(items[-2:],".")
  if filter(lambda x: x not in "0123456789",string.join(items,""))=="":
    return fqdn # Was an IP
  else:
    print "dunno",fqdn
    return string.join(items[-3:],".")

def doTraceroute(tracerouteTo,outputWin):
  # We use popen2 so that we can kill off the process if it's taking
  # too long. Doing a close() on the read pipe provided by os.popen()
  # seems to wait until the process exits of its own accord. Using
  # popen2 and closing the write pipe appears to kill it off now.
  numStarLines=0
  readFrom,writeTo=popen2.popen2("%s 2>&1 %s" % (whichTraceroute,tracerouteTo))
  while 1:
    ready=select.select([readFrom],[],[],kTooLongForTracerouteLine)
    if len(ready[0])==0:
      writeTo.close()
      readFrom.close()
      outputWin.addLine("Taking too long, forget it")
      break
    line=readFrom.readline()
    if not line:
      break
    outputWin.addLine(line)
    if string.count(line,"*")==3:
      numStarLines=numStarLines+1
      if numStarLines==kTracerouteMaxStarLines:
        writeTo.close()
        readFrom.close()
        outputWin.addLine("Too many stars, forget it")
        break
  outputWin.addLine("Done")
  return None

# To hold some state
class menuCBO:
  def __init__(self,kind,text):
    self.kind=kind
    self.text=text
    return None

  def __call__(self):
    gLookupWin.lift()
    gLookupWin.doCommand(self.kind,self.text)
    return None

class clipboardCBO:
  def __init__(self):
    self.text=""
    return None

  def set(self,text):
    self.text=text
    return None

  def __call__(self,arg1Ignored,arg2Ignored):
    return self.text

class spamWin:
  def __init__(self,inFile):
    self.createWidgets(inFile)
    gTkRoot.title(kMyName)
    return None

  def createWidgets(self,inFile):
    gTkRoot.bind("<Key>",self.keyCB)
    st=Pmw.ScrolledText(gTkRoot,
      text_bg="white",
      #text_state="disabled",
      text_wrap="word",
      text_spacing1=5,
      text_font=kMyFont,
      vertscrollbar_activebackground=kTkDefaultBG,
      horizscrollbar_activebackground=kTkDefaultBG,
      )
    self.textArea=st.component("text")
    #textArea.configure(bg="white",wrap='word',spacing1=5) # Wrap but put some space between wrapped lines
    #st.component("vertscrollbar")['activebackground']=kTkDefaultBG
    #st.component("horizscrollbar")['activebackground']=kTkDefaultBG
    st.pack(side=TOP,fill=BOTH,expand=1)
    self.st=st
    #self.textArea=textArea
    self.fillText(inFile)
    self.textArea.configure(state=DISABLED)
    self.textArea.tag_bind("interesting","<Button-1>",self.leftMouseInterestingCB)
    self.textArea.tag_bind("interesting","<Button-3>",self.rightMouseInterestingCB)
    return None

  def leftMouseInterestingCB(self,event):
    global gMyClipboard
    clickIndex=self.textArea.index("@"+repr(event.x)+","+repr(event.y))
    allTagRanges=self.textArea.tag_ranges("interesting")
    for count in range(0,len(allTagRanges),2):
      if self.textArea.compare(allTagRanges[count],"<=",clickIndex) and \
        self.textArea.compare(clickIndex,"<=",allTagRanges[count+1]):
        break
    else:
      sys.stderr.write("range not found (can't happen)%s" % kWidgetLineSep)
      return

    self.textArea.tag_remove("mysel", "1.0", END)
    self.textArea.tag_add("mysel",allTagRanges[count],allTagRanges[count+1])
    self.textArea.tag_configure("mysel",background=kTkDefaultBG)
    clickedText=self.textArea.get(allTagRanges[count],allTagRanges[count+1])
    # I really, really hate the way tkinter handles the clipboard. Everything
    # bad about the way X does and then a bunch of new annoyances.
    gMyClipboard.set(clickedText)
    gTkRoot.selection_own()
    gTkRoot.selection_handle(gMyClipboard)
    return "break"

  def rightMouseInterestingCB(self,event):
    clickIndex=self.textArea.index("@"+repr(event.x)+","+repr(event.y))
    allTagRanges=self.textArea.tag_ranges("interesting")
    for count in range(0,len(allTagRanges),2):
      if self.textArea.compare(allTagRanges[count],"<=",clickIndex) and \
        self.textArea.compare(clickIndex,"<=",allTagRanges[count+1]):
        break
    else:
      sys.stderr.write("range not found (can't happen)%s" % kWidgetLineSep)
      return

    clickedRange=(allTagRanges[count],allTagRanges[count+1])
    clickedText=self.textArea.get(allTagRanges[count],allTagRanges[count+1])
    m=Menu(gTkRoot,tearoff=0,activebackground=kTkDefaultBG)
    m.add_command(label="whois",command=menuCBO("whois",clickedText))
    m.add_command(label="nslookup",command=menuCBO("nslookup",clickedText))
    m.add_command(label="mx lookup",command=menuCBO("mx lookup",clickedText))
    m.add_command(label="nslookup, whois",command=menuCBO("nslookup, whois",clickedText))
    m.add_command(label="traceroute",command=menuCBO("traceroute",clickedText))
    m.post(event.x_root,event.y_root)
    # Delay to ignore the first mouse-up so that menu stays visible when
    # user releases mouse click that opened it. There's prolly a better
    # way.
    gTkRoot.after(250,m.grab_set)
    return "break"

  def fillText(self,inFile):
    lines=inFile.readlines()
    inFile.close()
    whereFrom=None
    whereReplyTo=None

    # Pass 1
    count=0
    while count<len(lines):
      if lines[count]==kFileLineSep:
        break

      if lines[count][0] in string.whitespace:
        if count==0:
          raise CantHappenError("First received header starts with whitespace")
        lines[count-1]=lines[count-1][:-1]+" "+string.lstrip(lines[count])
        del lines[count]
      else:
        if leftMatchCI(lines[count],"from: "):
          whereFrom=count
        elif leftMatchCI(lines[count],"reply-to: "):
          whereReplyTo=count
        count=count+1

    # Pass 2
    tclLineNo=0
    inHeaders=1
    for l in lines:
      tclLineNo=tclLineNo+1 # Tcl counts lines from 1 and characters from 0. Feh.

      self.textArea.insert(END,l)

      # If there are hex escapes, unescape them, stuff the unescaped line in with
      # a yellow background, and proceed using the unescaped version
      if not inHeaders:
        unHexedLine=hexEscapeRE.sub(lambda matchObj: chr(hexStrToDecVal(matchObj.group(1))),l)
        if unHexedLine<>l:
          tclLineNo=tclLineNo+1
          self.textArea.insert(END,unHexedLine)
          self.textArea.tag_add("unhexed","%i.0" % tclLineNo, "%i.end" % tclLineNo)
          l=unHexedLine

      if not inHeaders:
        unDecLine=decEscapeRE.sub(lambda matchObj: chr(int(matchObj.group(1))),l)
        if unDecLine<>l:
          tclLineNo=tclLineNo+1
          self.textArea.insert(END,unDecLine)
          self.textArea.tag_add("unhexed","%i.0" % tclLineNo, "%i.end" % tclLineNo)
          l=unDecLine

      if l==kFileLineSep and inHeaders:
        inHeaders=0
      elif inHeaders:
        if leftMatchCI(l,"received: "):
          highlightRuns=self.getReceivedHighlights(l)
          for run in highlightRuns:
            self.textArea.tag_add("interesting","%i.%i" % (tclLineNo,run[0]),"%i.%i" % (tclLineNo,run[1]))
        elif tclLineNo-1==whereReplyTo or (tclLineNo-1==whereFrom and whereReplyTo==None):
          m=kInterestingStuffREs['emailDomain'].search(l)
          if m<>None:
            span=m.span(1)
            rangeStart="%i.%i" % (tclLineNo,span[0])
            rangeEnd="%i.%i" % (tclLineNo,span[1])
            self.textArea.tag_add("interesting",rangeStart,rangeEnd)
      else: # In body
        for thisRE in kInterestingStuffREs.values():
          m=thisRE.search(l)
          while m<>None:
            span=m.span(1)
            rangeStart="%i.%i" % (tclLineNo,span[0])
            rangeEnd="%i.%i" % (tclLineNo,span[1])
            self.textArea.tag_add("interesting",rangeStart,rangeEnd)
            startAt=span[1]
            m=thisRE.search(l,startAt)

    self.textArea.tag_config("interesting",foreground=kInterestingForeground)
    self.textArea.tag_config("unhexed",background=kUnHexedBackground)
    return None


  def getReceivedHighlights(self,line):
    highlightRuns=[]

    heloResult=heloRE.search(line)
    if heloResult<>None:
      highlightRuns.append(heloResult.span(1))

    byResult=byRE.search(line)
    if byResult<>None:
      highlightRuns.append(byResult.span(1))

    parensResult=parensRE.search(line)
    if parensResult<>None:
      parensStart=parensResult.span(0)[0]+1
      parensText=parensResult.group(1)
      reverseLookupResult=reverseLookupRE.search(parensText)
      if reverseLookupResult<>None and reverseLookupResult.group(0)<>"unknown" \
        and reverseLookupResult.group(0)<>"":
        mySpan=reverseLookupResult.span(0)
        highlightRuns.append((mySpan[0]+parensStart,mySpan[1]+parensStart))

      receivedIPResult=ipRE.search(parensText)
      if receivedIPResult<>None:
        mySpan=receivedIPResult.span(1)
        highlightRuns.append((mySpan[0]+parensStart,mySpan[1]+parensStart))
    return highlightRuns

  def keyCB(self,event):
    if event.char=="w":
      self.close()
    elif event.char==" ":
      self.st.yview("scroll","1","pages")
    elif event.keysym=="Left":
      self.st.xview("scroll","-2","units")
    elif event.keysym=="Right":
      self.st.xview("scroll","2","units")
    elif event.keysym=="Up":
      self.st.yview("scroll","-1","units")
    elif event.keysym=="Down":
      self.st.yview("scroll","1","units")
    return "break"

  def close(self):
    sys.exit(0)

def main():
  global gTkRoot
  global gMyClipboard
  global gLookupWin

  DNS.ParseResolvConf()
  gTkRoot=Tk()
  Pmw.initialise(gTkRoot)
  gMyClipboard=clipboardCBO()
  gLookupWin=lookupWin()
  gTkRoot.selection_handle(gMyClipboard)
  gTkRoot.geometry("%dx%d%+d%+d" % (600,300,10,10))
  inFile=sys.stdin
  spamWin(inFile)
  gTkRoot.mainloop()
  return None

if __name__=='__main__':
  main()
